/*
 * Collisions 5000 Info - a logic puzzle game for PalmOS devices.
 * 
 * frmMain.c ,v 1.00 2001/05/31 
 * Copyright (C) 2001, Graham Rowbottom
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 
#include <PalmOS.h>
#include "Collisions5000.h"
#include "Collisions5000_res.h"   
#include "Bitmap.h"





/***********************************************************************
*
*	handleLevelFinished
*
*	called when the level is completed. tidies up any remaining shots
*	and brings up the level completed screen
*
*	RETURNS: nothing
*
***********************************************************************/
void handleLevelFinished()
{
	destroyRemainingShots();
	FrmPopupForm(frmLevFinished);
}

/***********************************************************************
*
*	destroyRemainingShots
*
*	deallocate the memory from any shots currently in play
*	
*
*	RETURNS: nothing
*
***********************************************************************/
void destroyRemainingShots()
{
	int m, x;
	
	if (gameBullets != NULL)
	{
		m = gameBullets->numShots;
		for (x=0; x<m; x++)
			removeShotFromBulletList(0, gameBullets);
	}
	
	if (nextTurnAdd != NULL)
	{	
		m = nextTurnAdd->numShots;
		for (x=0; x<m; x++)
			if (nextTurnAdd->laserShot[0] != NULL)
				removeShotFromBulletList(0, nextTurnAdd);
	}
}

/***********************************************************************
*
*	moveBullet
*
*	moves the parameter bullet one square in its designated direction
*	ASSUMES: parameter is not null (checked in Tick());
*
*	RETURNS: nothing
*
***********************************************************************/
void moveBullet(objectType *thisShot)
{
	
	switch(thisShot->direction)
	{
		case LEFT:
			thisShot->position.x--;
			break;
		case RIGHT:
			thisShot->position.x++;
			break;
		case UP:
			thisShot->position.y--;
			break;
		case DOWN:
			thisShot->position.y++;
			break;
	}
}


/***********************************************************************
*
*	handleClick
*
*	implements behaviour corresponding to a user press on the digitiser
*	
*	PARAMS: cartesian co-ords of screen press
*	RETURNS: nothing
*
***********************************************************************/
void handleClick(int x, int y)
{
	objectType * thisSquare;
	int sx, sy;
	sx = (x-X_ORIGIN)/CELL_X_WIDTH;
	sy = (y-Y_ORIGIN)/CELL_Y_WIDTH;
	
	thisSquare = gameState->level->square[sx][sy];
	if (thisSquare != NULL)
		switch(thisSquare->type)
		{
			case TRACK_MOVE:
				moveObjectIfPossible(thisSquare->extras.trackArrow.trackGun, thisSquare->direction);
				break;
			
			case TRACK_GUN:
			case POD_GUN:
			{
				int nx = thisSquare->position.x;
				int ny = thisSquare->position.y;
				switch(thisSquare->direction)
				{
					case RIGHT:	
						nx++;
						break;
					case LEFT:
						nx--;
						break;
					case UP:
						ny--;
						break;
					case DOWN:
						ny++;
				}
				
				addShotToBulletList(create_object(LASER_SHOT, nx, ny, thisSquare->direction), gameBullets);
			}
			break;
						
		}
}

/***********************************************************************
*
*	redrawLastLocation
*
*	redraws the square through which a shot just passed.
*   called only from tick()  
*
*	PARAMS: The shot in question
*	RETURNS: nothing
*
***********************************************************************/
void redrawLastLocation(objectType *thisShot)
{
	int x = thisShot->extras.bullet.oldPosition.x;
	int y = thisShot->extras.bullet.oldPosition.y;
	
	objectType * thisSquare = gameState->level->square[x][y];
	
	if (thisSquare == NULL)
	{
		drawEmptyRectangle(X_ORIGIN+x*CELL_X_WIDTH, Y_ORIGIN+y*CELL_Y_WIDTH);
	}
	else
	{
		drawSquare(thisSquare, x, y);
	}
}


/***********************************************************************
*
*	updateCurrentLocation
*
*	update (logically and visually) the state of a square into which a shot just entered
*	
*	PARAM: the shot in question
*	RETURNS: whether the shot is still 'alive'
*
***********************************************************************/
Boolean updateCurrentLocation(objectType *thisShot)
{
	int i, x, y;
	int oldDir;
	Boolean stillAlive = true;
	objectType *thisSquare;
	
	x = thisShot->position.x;
	y = thisShot->position.y;
	thisSquare = gameState->level->square[x][y];
	
	if (thisSquare != NULL)
	{
		
		switch(thisSquare->type)
		{
		
			case PORTAL:
				switch (thisSquare->direction)
				{
					case CLOSED:
						thisSquare->direction = OPEN;
						drawShot(thisShot);
						break;
					case OPEN:
						gameState->levelFinished = true;
						handleLevelFinished();
						break;
				}
				stillAlive = false;
				break;
			
			case TRACK:
			case ACID_BLOB:
				moveObjectIfPossible(thisSquare, thisShot->direction);
				stillAlive = false;
				break;
			
			case ROTATE:
				switch (thisSquare->direction)
				{
					case HORIZONTAL:
						switch (thisShot->direction)
						{
							case RIGHT:
							case LEFT:
								thisSquare->direction = VERTICAL;
								break;
							default:
								stillAlive = false;
								break;
						}
						break;	
		
					case VERTICAL:
						switch (thisShot->direction)
						{
							case UP:
							case DOWN:
								thisSquare->direction = HORIZONTAL;
								break;
							default:
								stillAlive = false;
								break;
						}
						break;
				}
				break;
		
			case BROKEN_BRICK:
				MemPtrFree(thisSquare);
				gameState->level->square[x][y] = NULL;
				drawSquare(NULL, x, y);
				stillAlive = false;
				break;
		
			case BRICK_SQUARE:
				thisSquare->type = BROKEN_BRICK;
				drawSquare(thisSquare, x, y);
				stillAlive = false;
				break;
			
			case ONE_SHOT:
				switch(thisSquare->direction)
				{
					case HORIZONTAL:
						if (thisShot->direction == LEFT || thisShot->direction == RIGHT)
							thisSquare->type = SOLID_SQUARE;
						else
							stillAlive = false;
						break;
					case VERTICAL:		
						if (thisShot->direction == UP || thisShot->direction == DOWN)
							thisSquare->type = SOLID_SQUARE;
						else
							stillAlive = false;
						break;
				}
				break;
			
			case TRANS_MIRROR:
				oldDir = thisShot->direction;
				
			case NORMAL_MIRROR:
				if (!resolveReflection(thisSquare->direction, &thisShot->direction))
				{	
					moveObjectIfPossible(gameState->level->square[x][y], thisShot->direction);
					stillAlive = false;
				}
				else
				{
					if (thisSquare->type == TRANS_MIRROR)
					{
						int nx = thisSquare->position.x;
						int ny = thisSquare->position.y;
						
						switch (thisSquare->direction)
						{
							case UP_LEFT:
								if (thisShot->direction == UP)
									nx++;
								else 
									ny++;
								break;
							case UP_RIGHT:
								if (thisShot->direction == UP)
									nx--;
								else
									ny++;
								break;
							case DOWN_RIGHT:
								if (thisShot->direction == DOWN)
									nx--;
								else
									ny--;
								break;
							case DOWN_LEFT:
								if (thisShot->direction == DOWN)
									nx++;
								else
									ny--;
								break;
						}
						
						if (nextTurnAdd->numShots + gameBullets->numShots < MAX_SHOTS-1) 
						addShotToBulletList(create_object(LASER_SHOT, nx, ny, oldDir), nextTurnAdd); //oldDir set by TRANS_MIRROR case									
					}
				}
				break;
			
			case SOLID_SQUARE:
				stillAlive = false;
				break;
				
			case POD_GUN:
				if (++thisSquare->direction > 3)
					thisSquare->direction = 0;
					
				drawShot(thisShot);
				stillAlive = false;
				break;
				
			default:
				stillAlive = false;
				break;
		}
	}
	
	return stillAlive;
}


/***********************************************************************
*
*	drawShot
*
*	render the square that the given shot now occupies. primary use is
*	for drawing 'elbow' on mirror reflections
*
*	RETURNS: nothing
*
***********************************************************************/
void drawShot(objectType * thisShot)
{
	int x = thisShot->position.x;
	int y = thisShot->position.y;
	Int16 rsc;
	objectType * thisSquare = gameState->level->square[x][y];
	x = X_ORIGIN+x*CELL_X_WIDTH;
	y = Y_ORIGIN+y*CELL_Y_WIDTH;
	
	
	if (thisSquare == NULL)
		switch(thisShot->direction)
		{
			case RIGHT:
			case LEFT:
				drawBitmap(CURRENT_HORIZ_SHOT, x, y);
				break;
			case UP:
			case DOWN:
				drawBitmap(CURRENT_VERT_SHOT, x, y);
				break;
		}
	else
	{
		switch(thisSquare->type)
		{
			case NORMAL_MIRROR:
				switch (thisSquare->direction)
				{
					case UP_LEFT:
						rsc = CURRENT_N_MIRROR_UL_SHOT;
						break;
					case UP_RIGHT:
						rsc = CURRENT_N_MIRROR_UR_SHOT;
						break;
					case DOWN_LEFT:
						rsc = CURRENT_N_MIRROR_DL_SHOT;
						break;
					case DOWN_RIGHT:
						rsc = CURRENT_N_MIRROR_DR_SHOT;
						break;
				}
			break;
			
			case TRANS_MIRROR:
				switch (thisSquare->direction)
				{
					case UP_LEFT:
						rsc = CURRENT_N_MIRROR_UL_TRANS_SHOT;
						break;
					case UP_RIGHT:
						rsc = CURRENT_N_MIRROR_UR_TRANS_SHOT;
						break;
					case DOWN_LEFT:
						rsc = CURRENT_N_MIRROR_DL_TRANS_SHOT;
						break;
					case DOWN_RIGHT:
						rsc = CURRENT_N_MIRROR_DR_TRANS_SHOT;
						break;
				}
			break;
			
			default:
				drawSquare(thisSquare, thisShot->position.x, thisShot->position.y);
				rsc = 0;
				break;
		}
		
		if (rsc != 0)
			drawBitmap(rsc, x, y);
		
	}
}


/***********************************************************************
*
*	resolveReflection
*
*	'reflect' if possible a shot with the given direction and mirror direction
*	
*	PARAM: mirror direction and shot direction reference
*	RETURNS: whether a reflection was possible
*
***********************************************************************/
Boolean resolveReflection(unsigned short mirrorDir, unsigned short *shotDir)
{
	switch (mirrorDir)
	{
		case UP_LEFT:
			switch (*shotDir)
			{
				case RIGHT:
					*shotDir = UP;
					break;
				case DOWN:
					*shotDir = LEFT;
					break;
				default:
					return false;
			}
			break;
		case UP_RIGHT:
			switch (*shotDir)
			{
				case LEFT:
					*shotDir = UP;
					break;
				case DOWN:
					*shotDir = RIGHT;
					break;
				default:
					return false;
			}
			break;
		case DOWN_LEFT:
			switch (*shotDir)
			{
				case UP:
					*shotDir = LEFT;
					break;
				case RIGHT:
					*shotDir = DOWN;
					break;
				default:
					return false;
			}
			break;
		case DOWN_RIGHT:
			switch (*shotDir)
			{
				case LEFT:
					*shotDir = DOWN;
					break;
				case UP:
					*shotDir = RIGHT;
					break;
				default:
					return false;
			}
			break;
	}
	
	return true;			
}

/***********************************************************************
*
*	moveObjectIfPossible
*
*	attempt to move the passed object in the given direction. Updates the display
*	and logic. Takes into account behaviour of acid and track guns
*	
*	PARAM: Object being moved
*	RETURNS: Whether the object was moved
*
***********************************************************************/
Boolean moveObjectIfPossible(objectType *obj, unsigned short dir)
{
	objectType * targetSquare;
	int targetx = obj->position.x;
	int targety = obj->position.y;
	
	switch (dir)
	{
		case LEFT:
			targetx--;
			break;
		case RIGHT:
			targetx++;
			break;
		case UP:
			targety--;
			break;
		case DOWN:
			targety++;
			break;
	}
	
	targetSquare = gameState->level->square[targetx][targety];
	if (targetSquare == NULL)
	{
		// can move the tracks of a track gun...
		if (obj->type != TRACK_GUN)
		{
			// redraw the empty 'from' square
			gameState->level->square[obj->position.x][obj->position.y] = NULL;
			drawSquare(NULL, obj->position.x, obj->position.y);
			
			// update and draw new 'to' square
			obj->position.x = targetx;
			obj->position.y = targety;
			gameState->level->square[targetx][targety] = obj;
			drawSquare(obj, targetx, targety);
		}		
		return true;
	}
	// if we move acid into something or something into acid...
	else if (targetSquare->type == ACID_BLOB || obj->type == ACID_BLOB)
	{
		// as long as we're not talking about the border squares
		if (targetx<MAX_X_SQUARES-1 && targetx>0 && targety>0 && targety<MAX_Y_SQUARES-1)
		{
			// free up object that moved into acid
			gameState->level->square[obj->position.x][obj->position.y] = NULL;
			drawSquare(NULL, obj->position.x, obj->position.y);
			MemPtrFree(obj);
			
			// free up acid itself
			MemPtrFree(targetSquare);
			gameState->level->square[targetx][targety] = NULL;
			drawSquare(NULL, targetx, targety);
			
			return true;
		}		
	}
	else if (obj->type == TRACK_GUN)
	{
		if (targetSquare->type == TRACK)
		{
			// check that it's a valid bit of track (a wrong one could
			// have been moved there)
			switch (targetSquare->direction)
			{
				case HORIZONTAL:
					if (obj->direction == LEFT || obj->direction == RIGHT)
						return false;
					break;
				case VERTICAL:
					if (obj->direction == UP || obj->direction == DOWN)
						return false;
					break;
			}
			
			swapSquares(&gameState->level->square[obj->position.x][obj->position.y],	&gameState->level->square[targetx][targety]);	
						
			drawSquare(obj, obj->position.x, obj->position.y);
			drawSquare(targetSquare, targetSquare->position.x, targetSquare->position.y);
		}
		else
			return false;
	}
	
	return false;
}

void swapSquares(objectType ** a, objectType ** b)
{
	if (a!= NULL && b!= NULL)
	{
		objectType * c;
		int d;
		
		c = *a;
		*a = *b;
		*b = c;
		
		d = (*a)->position.x;
		(*a)->position.x = (*b)->position.x;
		(*b)->position.x = d;
		
		d = (*a)->position.y;
		(*a)->position.y = (*b)->position.y;
		(*b)->position.y = d;
		
	}
}



Boolean frmMain_HandleEvent(EventPtr event)
{
	FormPtr form;
	Boolean handled = false;

	switch (event->eType)
	{
		case menuCmdBarOpenEvent:
			if (gameState != NULL)
			gameState->playing = false;
			break;
		
		case penDownEvent:
			if (gameState!= NULL)
			if (!gameState->paused)
			handleClick(event->screenX, event->screenY);
			break;
		case frmOpenEvent:
			// Repaint form on open
			form = FrmGetActiveForm();
			FrmDrawForm(form);
			drawScreen();
			
			handled = true;
			break;
		default:
			break;
	}

	return handled;
}

/***********************************************************************
*
*	addShotToBulletList
*
*	add a shot to a bullet list
*
*	
*	RETURNS: whether the bullet was added successfully
*
***********************************************************************/
Boolean addShotToBulletList(objectType * newShot, bulletsType * list)
{
	if (list != NULL)
	if (list->numShots < MAX_SHOTS-1)
	{
		list->laserShot[list->numShots] = newShot;
		list->numShots++;
		return true;
	}
	
	return false;
}


/***********************************************************************
*
*	removeShotFromBulletList
*
*	removes the given index from the bullet list
*
*	RETURNS: whether the removal was successful (index valid)
*
***********************************************************************/
Boolean removeShotFromBulletList(int index, bulletsType * list)
{
	if (index < list->numShots && index >=0)
	{
		int i;
		MemPtrFree(list->laserShot[index]);
		list->laserShot[index] = NULL;
		list->numShots--;
		
		for (i=index; i<list->numShots; i++)
			list->laserShot[i] = list->laserShot[i+1];
			
		// this is so tick() can run the right number of shots
		// in a cycle
		list->laserShot[list->numShots+1] = NULL;
		return true;
	}
	
	return false;
}


/***********************************************************************
*
*	create_object
*
*	allocates memory and sets up an object with the given parameters
*
*	PARAMS: object type, start position, direction
*	RETURNS: reference to the object. NULL if failed
*
***********************************************************************/
objectType* create_object(int type, int startx, int starty, int dirn)
{
	objectType * newShot = MemPtrNew(sizeof(objectType));
	
	if (newShot != NULL)
	{
		newShot->type = type;
		newShot->direction = dirn;
		newShot->position.x = startx;
		newShot->position.y = starty;
		
		switch (type)
		{
			case LASER_SHOT:
				newShot->extras.bullet.oldPosition.x = startx;
				newShot->extras.bullet.oldPosition.y = starty;
				newShot->extras.bullet.justCreated = true;
				break;
				
		}
	}
	
	return newShot;
}




/***********************************************************************
*
*	drawSquare
*
*	Renders the given square onto the screen. Normally the square contains
*	the necessary position information, but empty squares are represented as
*	null so the position is also passed
*
*	RETURNS: nothing
*
***********************************************************************/
void drawSquare(objectType *thisSquare, int x, int y)
{
	int type, dir;
	Int16 grr;
		
	if (thisSquare != NULL)
	{
		type = thisSquare->type;
		dir = thisSquare->direction;
	}
	else
	{	
		drawEmptyRectangle(X_ORIGIN+x*CELL_X_WIDTH, Y_ORIGIN+y*CELL_Y_WIDTH);
		return;
	}
			
	switch (type)
	{
		case PORTAL:
			switch(dir)
			{
				case OPEN:
					grr = CURRENT_PORTAL_OPEN;
					break;
				case CLOSED:
					grr = CURRENT_PORTAL_CLOSED;
					break;
			}
			break;
			
		case TRACK_GUN:
			switch(dir)
			{
				case LEFT:
					grr = CURRENT_TRACK_GUN_LEFT;
					break;
				case RIGHT:
					grr = CURRENT_TRACK_GUN_RIGHT;
					break;
				case UP:
					grr = CURRENT_TRACK_GUN_UP;
					break;
				case DOWN:
					grr = CURRENT_TRACK_GUN_DOWN;
					break;
			}
			break;
			
		case TRACK:
			switch (dir)
			{
				case HORIZONTAL:
					grr = CURRENT_TRACK_HORIZ;
					break;
				case VERTICAL:
					grr = CURRENT_TRACK_VERT;
					break;
			}
			break;
			
		case TRACK_MOVE:
			switch(dir)
			{
				case LEFT:
					grr = CURRENT_TRACK_MOVE_LEFT;
					break;
				case RIGHT:
					grr = CURRENT_TRACK_MOVE_RIGHT;
					break;
				case UP:
					grr = CURRENT_TRACK_MOVE_UP;
					break;
				case DOWN:
					grr = CURRENT_TRACK_MOVE_DOWN;
					break;
			}
			break;
				
		case ACID_BLOB:
			grr = CURRENT_ACID;
			break;
			
		case BRICK_SQUARE:
			grr = CURRENT_BRICK;
			break;
			
		case BROKEN_BRICK:
			grr = CURRENT_BROKEN_BRICK;
			break;
			
		case ROTATE:
			switch (dir)
			{
				case HORIZONTAL:
					grr = CURRENT_ROTATE_HORIZ;
					break;
				case VERTICAL:
					grr = CURRENT_ROTATE_VERT;
					break;
			}
			break;
		
		case NORMAL_MIRROR:
			switch(dir)
			{
				case UP_LEFT:
					grr = CURRENT_N_MIRROR_UL;
					break;
				case UP_RIGHT:
					grr = CURRENT_N_MIRROR_UR;
					break;
				case DOWN_LEFT:
					grr = CURRENT_N_MIRROR_DL;
					break;
				case DOWN_RIGHT:
					grr = CURRENT_N_MIRROR_DR;
					break;
			}
		break;
		
		case TRANS_MIRROR:
			switch(dir)
			{
				case UP_LEFT:
					grr = CURRENT_N_MIRROR_UL_TRANS;
					break;
				case UP_RIGHT:
					grr = CURRENT_N_MIRROR_UR_TRANS;
					break;
				case DOWN_LEFT:
					grr = CURRENT_N_MIRROR_DL_TRANS;
					break;
				case DOWN_RIGHT:
					grr = CURRENT_N_MIRROR_DR_TRANS;
					break;
			}
		break;
		
		case SOLID_SQUARE:
			grr = CURRENT_SOLID;
			break;
			
		case ONE_SHOT:
			switch(dir)
			{
				case HORIZONTAL:
					grr = CURRENT_ONE_SHOT_HORIZ;
					break;
				case VERTICAL:
					grr = CURRENT_ONE_SHOT_VERT;
					break;
			}
			break;
		
		case POD_GUN:
			switch (dir)
			{
				case RIGHT:
					grr = CURRENT_POD_RIGHT;
					break;
				case LEFT:
					grr = CURRENT_POD_LEFT;
					break;
				case UP:
					grr = CURRENT_POD_UP;
					break;
				case DOWN:
					grr = CURRENT_POD_DOWN;
					break;
			}
			break;
		default:
			grr = 0;
			break;
	}
	
	if (grr != 0)
	{		
		drawBitmap(grr, X_ORIGIN+x*CELL_X_WIDTH, Y_ORIGIN+y*CELL_Y_WIDTH);
	}
	else
	{
		drawEmptyRectangle(X_ORIGIN+x*CELL_X_WIDTH, Y_ORIGIN+y*CELL_Y_WIDTH);
	}		
}

/***********************************************************************
*
*	drawScreen
*
*	renders the current level onto the screen using the current
*	game dimensions
*
*	RETURNS: nothing
*
***********************************************************************/
void drawScreen()
{
	int x, y;
	Word grr;
	FormPtr form = FrmGetActiveForm();
	int type, dir;
	BitmapPtr *bitmap;
	MemHandle resH;
	
	WinSetDrawWindow(FrmGetWindowHandle(form));
	
	/*for (x=0; x<MAX_X_SQUARES; x++)
		for (y=0; y<MAX_Y_SQUARES; y++)
		{
			drawSquare(gameState->level->square[x][y], x, y);
		}*/
		
	for (x=0; x<MAX_X_SQUARES; x+=2)
		for (y=0; y<MAX_Y_SQUARES; y++)
		{
			drawSquare(gameState->level->square[x][y], x, y);
		}
		
	for (x=MAX_X_SQUARES-1; x>=0; x-=2)
		for (y=0; y<MAX_Y_SQUARES; y++)
		{
			drawSquare(gameState->level->square[x][y], x, y);
		}

}

/***********************************************************************
*
*	drawEmptyRectangle
*
*	renders and empty square onto the screen at the given coords
*
*	RETURNS: nothing
*
***********************************************************************/
void drawEmptyRectangle(int x, int y)
{
	RectangleType rect;
	//FormPtr form = FrmGetActiveForm();
	//WinSetDrawWindow(FrmGetWindowHandle(form));
	
	RctSetRectangle(&rect, x, y, CELL_X_WIDTH, CELL_Y_WIDTH);
	WinEraseRectangle(&rect, 0);
	
	//return;
}

/***********************************************************************
*
*	drawBitmap
*
*	renders a bitmap with the given resource ID at the given position
*
*	RETURNS: nothing
*
***********************************************************************/
void drawBitmap(Int16 resID, Int16 x, Int16 y)
{
	MemHandle		resH;
	BitmapPtr	resP;


	resH = DmGetResource( bitmapRsc, resID );
	ErrFatalDisplayIf( !resH, "Missing bitmap" );
	resP = MemHandleLock(resH);
	WinDrawBitmap (resP, x, y);
	MemPtrUnlock(resP);
	DmReleaseResource( resH );
}


/***********************************************************************
*
*	TimUntilNextPeriod
*
*	calculates the time remaining until the next game 'tick' must be run
*
*	Returns: time (ticks) until the next period is pending
*
***********************************************************************/
static Int32 TimeUntillNextPeriod (void)
{
	Int32 timeRemaining;
	
	if (gameState->paused)
		return evtWaitForever;
		
	timeRemaining = gameState->nextPeriodTime - TimGetTicks();
	
	if (timeRemaining < 0)
	{
		timeRemaining = 0;
		gameState->nextPeriodTime = TimGetTicks();
	}
		
	return timeRemaining;
}

static int StartApplication(void)
{
	Boolean fatalError = true;
	// is the levels DB installed?
	if (DatabaseAccess_openDB(LEVELS_DB_NAME, LEVELS_DB_TYPE, &m_db))
	{
		if (DmNumRecords(m_db) > 0)
		{
			fatalError = false;
		}
		DmCloseDatabase(m_db);
	}
	
	if (fatalError)
	{
		FrmCustomAlert(alError, "You must install the levels .pdb file that came with the installation package", "", "");
		return -1;
	}
		
		
	
	gameState = MemPtrNew(sizeof(gameStateType));
	gameBullets = NULL;
	nextTurnAdd = NULL;
	globalLevelsContainer = NULL;
	// attemp to load the unfinished level that was in play when the application was last quit
	
	if (!DatabaseAccess_loadStatus())
	{
		// no level (either first run or level was finished at quit)
		// go to intro screen
		FrmGotoForm(frmIntro);
		gameState->paused = true;
		gameState->levelFinished = true;
		gameState->level = NULL;
	}
		
	
	gameState->nextPeriodTime = TimGetTicks()+30;
	gameState->pausedTime = TimGetTicks();
	return 0;
}

static Boolean ApplicationHandleEvent(EventPtr event)
{
	UInt16 formID;
	FormPtr form;
	Boolean handled = false;
	
	// Application event loop
	switch (event->eType)
	{
		case menuEvent:
			// Set event handler for application
			handled = ApplicationHandleMenu(event->data.menu.itemID);
			break;
		case frmLoadEvent:
			// Handle form load events
			formID = event->data.frmLoad.formID;
			form = FrmInitForm(formID);
			FrmSetActiveForm(form);		

			switch (formID)
			{
				case frmAbout:
					// Set event handler for frmAbout
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmAbout_HandleEvent);
					break;
				case frmLoadLevel:
					// Set event handler for frmLoadLevel
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmLoadLevel_HandleEvent);
					break;
				case frmIntro:
					// Set event handler for frmIntro
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmIntro_HandleEvent);
					break;
				case frmLevFinished:
					// Set event handler for frmLevFinished
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmLevFinished_HandleEvent);
					break;
				case frmMain:
					// Set event handler for frmMain
					FrmSetEventHandler(form, (FormEventHandlerPtr) frmMain_HandleEvent);
					break;
				default:
					break;
			}
			handled = true;
			break;
		default:
			break;
	}

	return handled;
}

static void EventLoop(void)
{
	UInt16 error;
	EventType event;

	// Main event loop
	do
	{
		// Get next event
		EvtGetEvent(&event, TimeUntillNextPeriod());
		
		if (event.eType == winExitEvent)
			{
			if (event.data.winExit.exitWindow == (WinHandle) FrmGetFormPtr(frmMain))
				{
				gameState->paused = true;
				gameState->pausedTime = TimGetTicks();
				}
			}

		// Detect entering the game's window.  Resume drawing to our window.
		else if (event.eType == winEnterEvent)
		{
			// In the current code, the menu doesn't remove itself when it receives
			// a winExitEvent.  There also isn't a call to see if the menu is visible.
			// For now we make our own MenuGetVisible and check it each time our
			// BoardView is entered.
			if (event.data.winEnter.enterWindow == (WinHandle) FrmGetFormPtr(frmMain) &&
				event.data.winEnter.enterWindow == (WinHandle) FrmGetFirstForm())
			{
				// Sometimes we can enter the game's window without knowing it was 
				// ever left.  In that case the pause time will not have been recorded.
				// Set the current period back to it's beginning
				if (!gameState->paused)
				{
					gameState->nextPeriodTime = TimGetTicks() + ADVANCE_TIME;
				}
				else
				{
					// Unpause the game.  Account for time lost during pause
					gameState->paused = false;
					gameState->nextPeriodTime += (TimGetTicks() - gameState->pausedTime + pauseLengthBeforeResumingInterruptedGame) ;
				}
			}
		}
		
		// If it's time, go to the next time period
		else if (TimeUntillNextPeriod() == 0 && !gameState->paused && !gameState->levelFinished)
		{
			tick();
		}
	
		
		// Handle event
		if (!SysHandleEvent(&event))
		{
			if (!MenuHandleEvent(0, &event, &error))
			{
				if (!ApplicationHandleEvent(&event))
					FrmDispatchEvent(&event);	
			}
		}
	}
	while (event.eType != appStopEvent);
}

static void StopApplication(void)
{
	DatabaseAccess_saveStatus();
	
	if (gameState != NULL)
		destroyApp();
		
	FrmCloseAllForms();
}

static Boolean save_OnSelect(UInt16 menuID)
{
	// Insert code for save
	DatabaseAccess_saveLevel(gameState->level, LEVELS_DB_NAME, LEVELS_DB_TYPE);
	return true;
}

static Boolean loadlevel_OnSelect(UInt16 menuID)
{
	FrmPopupForm(frmLoadLevel);
	return true;
}

/****************************************************************************/

UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
	int error;
	
	if (cmd == sysAppLaunchCmdNormalLaunch)
 	{
		// Application start code
		error = StartApplication();
		if (error)
			return error;
		
		// Maintain event loop
		EventLoop();

		// Stop application 
		StopApplication();
	}

	return 0;
}

/***********************************************************************
*
*	tick
*
*	update the game state
*	move each bullet one square, update the last square, update the current square
*
*	RETURNS: nothing
*
***********************************************************************/
void tick()
{
	int i, m;
	objectType *thisShot;
	
	if (nextTurnAdd->numShots > 0)
	{
		for (i=0; i<nextTurnAdd->numShots; i++)
		{
			addShotToBulletList(nextTurnAdd->laserShot[i], gameBullets);
			nextTurnAdd->laserShot[i] = NULL;
		}
		
		nextTurnAdd->numShots = 0;
	}
	
	
	for (i=0; i<gameBullets->numShots; i++)
	{
		thisShot = gameBullets->laserShot[i];
		//if (thisShot == NULL)
		//	continue;
			
		// don't draw last location of a new shot
		if (!thisShot->extras.bullet.justCreated)
			redrawLastLocation(thisShot);
		else
			thisShot->extras.bullet.justCreated = false;	
				
		// update the current location of this shot
		if (updateCurrentLocation(thisShot))
		{	
			// draw if still alive		
			drawShot(thisShot);
			
			// set up the last location of this shot
			thisShot->extras.bullet.oldPosition.x = thisShot->position.x;
			thisShot->extras.bullet.oldPosition.y = thisShot->position.y;			
			
			// move it ready for the next tick
			moveBullet(thisShot);
		}
		else
		{
			// if the shot 'died' in the update remove
			removeShotFromBulletList(i, gameBullets);
			i--;
		}		
	}
	
	gameState->nextPeriodTime += ADVANCE_TIME;
}

/****************************************************************************/

Boolean ApplicationHandleMenu(UInt16 menuID)
{
	Boolean handled = false;
	
	switch (menuID)
	{
		// Event handler for menu item
		case menuInstructions:
			handled = menuInstructions_OnSelect(menuID);
			break;
		// Event handler for menu item
		case mnuHint:
			handled = mnuHint_OnSelect(menuID);
			break;
		// Event handler for menu item
		case mnuAbout:
			handled = mnuAbout_OnSelect(menuID);
			break;
		// Event handler for menu item
		case mnuRestart:
			handled = mnuRestart_OnSelect(menuID);
			break;
		
		// Event handler for menu item
		case MnuLoadLevel:
			handled = loadlevel_OnSelect(menuID);
			break;
	
		default:
			break;
	}

	return handled;
}


/***********************************************************************
*
*	destroyApp
*
*	deallocate all the memory used by the game
*	
*
*	RETURNS: nothing
*
***********************************************************************/
void destroyApp()
{
	int x, m;
	destroyCurrentLevel();
	
	if (globalLevelsContainer != NULL)
	{
		int i;
		for (i=0; i<globalLevelsNumber; i++)
			MemPtrFree(globalLevelsContainer[i]);
				
		MemPtrFree(globalLevelsContainer);
	}
	
	destroyRemainingShots();
		
	if (gameBullets != NULL)
		MemPtrFree(gameBullets);	
	
	if (nextTurnAdd != NULL)
		MemPtrFree(nextTurnAdd);
	
	if (gameState != NULL)
		MemPtrFree(gameState);
	
	gameState = NULL;
	
	
}

/***********************************************************************
*
*	destroyCurrentLevel
*
*	deallocates all the memory used by this level
*	
*
*	RETURNS: nothing
*
***********************************************************************/
void destroyCurrentLevel()
{
	int x, y;
	if (gameState->level != NULL)
	{
		for (x=0; x<MAX_X_SQUARES; x++)
			for (y=0; y<MAX_Y_SQUARES; y++)
				if (gameState->level->square[x][y] != NULL)
					MemPtrFree(gameState->level->square[x][y]);
		
		MemPtrFree(gameState->level);
		gameState->level = NULL;
	}	
	
			
}

/***********************************************************************
*
*	setGameDimensions
*
*	Sets the current bitmap resource ids, screen size, cell size for each object type depending
*	on the current game dimension (large or small)
*
*	RETURNS: nothing
*
***********************************************************************/
void setGameDimensions(int mode)
{
	switch(mode)
	{
		case DIM_LARGE:
			MAX_X_SQUARES = 10;
			MAX_Y_SQUARES = 10;
			CELL_X_WIDTH = 16;
			CELL_Y_WIDTH = 16;
			X_ORIGIN = 0;
			Y_ORIGIN = 0;
			GAME_DIM = DIM_LARGE;
			break;
		
		case DIM_SMALL:
			MAX_X_SQUARES = 16;
			MAX_Y_SQUARES = 16;
			CELL_X_WIDTH = 10;
			CELL_Y_WIDTH = 10;
			X_ORIGIN = 0;
			Y_ORIGIN = 0;
			GAME_DIM = DIM_SMALL;
			break;
	}
	
	CURRENT_N_MIRROR_UL = fn_N_MIRROR_UL(GAME_DIM);
 	CURRENT_N_MIRROR_UR = fn_N_MIRROR_UR(GAME_DIM);
	CURRENT_N_MIRROR_DR = fn_N_MIRROR_DR(GAME_DIM);
    CURRENT_N_MIRROR_DL = fn_N_MIRROR_DL(GAME_DIM);
	CURRENT_N_MIRROR_UL_TRANS = fn_N_MIRROR_UL_TRANS(GAME_DIM);
 	CURRENT_N_MIRROR_UR_TRANS = fn_N_MIRROR_UR_TRANS(GAME_DIM);
	CURRENT_N_MIRROR_DR_TRANS = fn_N_MIRROR_DR_TRANS(GAME_DIM);
    CURRENT_N_MIRROR_DL_TRANS = fn_N_MIRROR_DL_TRANS(GAME_DIM);
	CURRENT_N_MIRROR_UL_SHOT = fn_N_MIRROR_UL_SHOT(GAME_DIM);
	CURRENT_N_MIRROR_UR_SHOT = fn_N_MIRROR_UR_SHOT(GAME_DIM);
	CURRENT_N_MIRROR_DR_SHOT = fn_N_MIRROR_DR_SHOT(GAME_DIM);
	CURRENT_N_MIRROR_DL_SHOT = fn_N_MIRROR_DL_SHOT(GAME_DIM);
	CURRENT_N_MIRROR_UL_TRANS_SHOT = fn_N_MIRROR_UL_TRANS_SHOT(GAME_DIM);
	CURRENT_N_MIRROR_UR_TRANS_SHOT = fn_N_MIRROR_UR_TRANS_SHOT(GAME_DIM);
	CURRENT_N_MIRROR_DR_TRANS_SHOT = fn_N_MIRROR_DR_TRANS_SHOT(GAME_DIM);
	CURRENT_N_MIRROR_DL_TRANS_SHOT = fn_N_MIRROR_DL_TRANS_SHOT(GAME_DIM);
	CURRENT_SOLID = fn_SOLID(GAME_DIM);
	CURRENT_HORIZ_SHOT = fn_HORIZ_SHOT(GAME_DIM);
	CURRENT_VERT_SHOT = fn_VERT_SHOT(GAME_DIM);
	CURRENT_POD_LEFT = fn_POD_LEFT(GAME_DIM);
	CURRENT_POD_UP = fn_POD_UP(GAME_DIM);
	CURRENT_POD_RIGHT = fn_POD_RIGHT(GAME_DIM);
	CURRENT_POD_DOWN = fn_POD_DOWN(GAME_DIM);
	CURRENT_ONE_SHOT_HORIZ = fn_ONE_SHOT_HORIZ(GAME_DIM);
	CURRENT_ONE_SHOT_VERT = fn_ONE_SHOT_VERT(GAME_DIM);
	CURRENT_ROTATE_HORIZ = fn_ROTATE_HORIZ(GAME_DIM);
	CURRENT_ROTATE_VERT = fn_ROTATE_VERT(GAME_DIM);
	CURRENT_ACID = fn_ACID(GAME_DIM);
	CURRENT_BRICK = fn_BRICK(GAME_DIM);
	CURRENT_BROKEN_BRICK = fn_BRICK_SHOT(GAME_DIM);
	CURRENT_TRACK_HORIZ = fn_TRACK_HORIZ(GAME_DIM);
	CURRENT_TRACK_VERT = fn_TRACK_VERT(GAME_DIM);
	CURRENT_TRACK_GUN_RIGHT = fn_TRACK_GUN_RIGHT(GAME_DIM);
	CURRENT_TRACK_GUN_LEFT = fn_TRACK_GUN_LEFT(GAME_DIM);
	CURRENT_TRACK_GUN_UP = fn_TRACK_GUN_UP(GAME_DIM);
	CURRENT_TRACK_GUN_DOWN = fn_TRACK_GUN_DOWN(GAME_DIM);
	CURRENT_TRACK_MOVE_RIGHT = fn_TRACK_MOVE_RIGHT(GAME_DIM);
 	CURRENT_TRACK_MOVE_LEFT = fn_TRACK_MOVE_LEFT(GAME_DIM);
	CURRENT_TRACK_MOVE_UP = fn_TRACK_MOVE_UP(GAME_DIM);
	CURRENT_TRACK_MOVE_DOWN = fn_TRACK_MOVE_DOWN(GAME_DIM);
	CURRENT_PORTAL_OPEN = fn_PORTAL_OPEN(GAME_DIM);
	CURRENT_PORTAL_CLOSED = fn_PORTAL_CLOSED(GAME_DIM);
	
}


/***********************************************************************
*
*	DatabaseAccess_openDB
*
*	opens a native database of given name and type
*	Creator is 'PGRR'
*
*	RETURNS: nothing
*
***********************************************************************/
Boolean DatabaseAccess_openDB(char * dbName, unsigned long dbType, DmOpenRef** db)
{
	Err			error = 0;
	DmOpenRef 	DBPtr = 0; // Define a pointer to the database
	Boolean		created = false;
	UInt16		cardNo = 0;
	UInt16		mode = dmModeReadWrite;
	LocalID dbID;
	
 	/* Find the database */
	if ((dbID = DmFindDatabase(cardNo, dbName)))
	{
		DBPtr = NULL;
		DBPtr = DmOpenDatabase(cardNo, dbID, mode);
		//if (DBPtr ==NULL)
		//	return true; 
	}
	//DBPtr = DmOpenDatabaseByTypeCreator( dbType, DBCreator, mode);
	
	/* If no database could be opened */									   
	if (!DBPtr) 
	{
		error = DmCreateDatabase( cardNo, dbName, DBCreator, dbType, created);
		/* If the DmCreateDatabase doesn't return 0 */
		if (error) 
			return false; 		
		DBPtr = DmOpenDatabaseByTypeCreator( dbType, DBCreator, mode);
		if (!DBPtr) 
			return (false);
		*db = DBPtr;
	}
	else
		*db = DBPtr;
		
	return true;
}

/***********************************************************************
*
*	DatabaseAccess_getLevelNames
*
*	returns the levelname of each level in the Levels Database
*	
*
*	RETURNS: success, filenames are returned in the 3rd param
*
***********************************************************************/
Boolean DatabaseAccess_getLevelnames(char *dbName, unsigned long dbType, char ***filenames, int *nRecs)
{
	int index;
	unsigned short MaxRec;
	MemHandle handle;
	char * strPtr;
	char ** _filenames;
	
	//DmOpenRef db;
	char * doc;
	
	if (DatabaseAccess_openDB(dbName, dbType, &m_db))
	{
		MaxRec = DmNumRecords(m_db);
		_filenames = (char **)MemPtrNew(sizeof(char *)*MaxRec);
		
		for (index = 0; index<MaxRec; index++)
		{
			handle = DmQueryRecord(m_db, index);
			doc = MemHandleLock(handle);
														
			_filenames[index] = (char *)MemPtrNew((StrLen(doc)+1));

			StrCopy(_filenames[index], doc);	
			MemHandleUnlock(handle);
			
		}
		
		*filenames = _filenames;
		*nRecs = MaxRec;
		DmCloseDatabase(m_db);
	}
	else
		return false;
	
	return true;
}


Boolean DatabaseAccess_deleteLevel(int index, char * name, unsigned long type)
{
	if (DatabaseAccess_openDB(name, type, &m_db))
	{
		if (index >=0 && index < DmNumRecords(m_db))
			DmDeleteRecord(m_db, index);
	}
}

/***********************************************************************
*
*	DatabaseAccess_saveLevel
*
*	saves the level in the 1st param to database
*	of specified name and type
*
*	RETURNS: success of operation
*
***********************************************************************/
Boolean DatabaseAccess_saveLevel(levelType * mlevel, char * name, unsigned long type)
{
	Boolean success = false;
	
	int x, y, index, levelSize, nonnull;
	MemHandle h;
	
	if (mlevel != NULL)
	{	
		if (DatabaseAccess_openDB(name, type, &m_db))
		{
			UInt16 index = dmMaxRecordIndex;

			nonnull = level_getNonNullSquares(mlevel);
			levelSize = (MAX_LEVEL_NAME_LENGTH) + nonnull*sizeof(objectType) ;
			
			//h = DmNewRecord(m_db, &index, levelSize);
			h = MemHandleNew(levelSize);
			
			if (h)
			{
				int actualSize, offset = 0;
				MemHandle recHandle;
				char * realRecPtr;
				char * recPtr = MemHandleLock(h);
				char * numsq = MemPtrNew(5);
				char * indexn = MemPtrNew(6);
				char * dim = MemPtrNew(5);
				char * startPtr = recPtr;
				
				StrCopy(recPtr, mlevel->levelName);
				
				recPtr +=StrLen(mlevel->levelName)+1;
					
				StrIToA(numsq, nonnull);
				//DmWrite(recPtr, recPtr, nonnull);
				StrCopy(recPtr, numsq);
				recPtr += StrLen(numsq)+1;
				
				StrIToA(indexn, mlevel->index); 
				StrCopy(recPtr, indexn);
				recPtr += StrLen(indexn)+1;
				
				StrIToA(dim, mlevel->dimension);
				StrCopy(recPtr, dim);
				recPtr += StrLen(dim)+1;
				
				StrCopy(recPtr, mlevel->levelHint);
				recPtr+=StrLen(mlevel->levelHint)+1;
				
				MemPtrFree(indexn);
				MemPtrFree(numsq);
				MemPtrFree(dim);
				
				// write objects (excluding the perimeter which is taken for granted)
				for (x = 1; x<MAX_X_SQUARES-1; x++)
					for (y=1; y<MAX_Y_SQUARES-1; y++)
						if (mlevel->square[x][y] != NULL)
						{
							saveLevel_saveSquare(mlevel->square[x][y], &recPtr);
						}	
						
						success = true;
				
				
				actualSize = recPtr - startPtr;
				recHandle = DmNewRecord(m_db, &index, actualSize);
				realRecPtr = MemHandleLock(recHandle);
				DmWrite(realRecPtr, 0, startPtr, actualSize);
				MemHandleUnlock(recHandle);		
				MemHandleUnlock(h);
				MemHandleFree(h);
				DmReleaseRecord(m_db, index, true);	
			}
			DmCloseDatabase(m_db);
		}
	
	}
	return success;
}

void saveLevel_saveSquare(objectType *thisSq, char **NrecPtr)
{
	char * recPtr = *NrecPtr;
	// write position
	char *dir = MemPtrNew(10);
	char *xpos = MemPtrNew(10);
	char *ypos = MemPtrNew(10);
	char *type = MemPtrNew(10);
	
	
	StrIToA(xpos, thisSq->position.x);
	StrIToA(ypos, thisSq->position.y);
	StrIToA(type, thisSq->type);
	StrIToA(dir, thisSq->direction);
	
	
	StrCopy(recPtr, xpos);
	recPtr+=StrLen(xpos)+1;
			
	StrCopy(recPtr, ypos);
	recPtr+=StrLen(ypos)+1;						
		
	StrCopy(recPtr, type);
	recPtr+=StrLen(type)+1;				
	
	// write direction
	StrCopy(recPtr, dir);
	recPtr+=StrLen(dir)+1;
							
	// write extras if any
	MemPtrFree(xpos);
	MemPtrFree(ypos);
	MemPtrFree(type);
	MemPtrFree(dir);
	
	switch (thisSq->type)
	{
		case TRACK_MOVE:
		{
			char * gxpos = MemPtrNew(4);
			char * gypos = MemPtrNew(4);
			
			gxpos = StrIToA(gxpos, thisSq->extras.trackArrow.trackGun->position.x);
			gypos = StrIToA(gypos, thisSq->extras.trackArrow.trackGun->position.y);
			
			StrCopy(recPtr, gxpos);
			recPtr+=StrLen(gxpos)+1;
			
			StrCopy(recPtr, gypos);
			recPtr+=StrLen(gypos)+1;
			
			MemPtrFree(gxpos);
			MemPtrFree(gypos);
		}	
			break;
									
	}
	
	*NrecPtr = recPtr;
}


/***********************************************************************
*
*	DatabaseAccess_loadStatus
*
*	used to load the state of the last unfinished level run when the app
*	was terminated. if there was no state saved, then does nothing.
*
*	RETURNS: nothing
*
***********************************************************************/
Boolean DatabaseAccess_loadStatus()
{
	if (DatabaseAccess_openDB(STATUS_DB_NAME, STATUS_DB_TYPE, &m_db))
	{
		if (DmNumRecords(m_db) > 0)
		{
			DmCloseDatabase(m_db);
			runLevel(STATUS_DB_NAME, STATUS_DB_TYPE, 0);
			return true;
		}
		else
		{
			DmCloseDatabase(m_db);
			return false;		
		}
	}
}

/***********************************************************************
*
*	DatabaseAccess_saveStatus
*
*	Saves the states of the current level held in the gameState (if unfinished) to a special
*	database so that it will be loaded straight away next time the app starts 
*
*	RETURNS: success
*
***********************************************************************/
Boolean DatabaseAccess_saveStatus()
{
	if (!gameState->levelFinished && gameState->level != NULL)
		if (DatabaseAccess_openDB(STATUS_DB_NAME, STATUS_DB_TYPE, &m_db))
		{
			if (DmNumRecords(m_db) > 0)
				DmRemoveRecord(m_db, 0);
				
			DmCloseDatabase(m_db);
			return DatabaseAccess_saveLevel(gameState->level, STATUS_DB_NAME, STATUS_DB_TYPE);
		}
	
	return false;
}

/***********************************************************************
*
*	DatabaseAccess_loadLevelIntoGameState
*
*	Loads the index record of the specified native database into gameState
*	
*
*	RETURNS: success of the operation
*
***********************************************************************/
Boolean DatabaseAccess_loadLevelIntoGameState(char *name, unsigned long type, int index)
//COPY the contents of this record into the gameState->level
{
	Boolean success = false;
	int x, y, i,levelSize, nonnull, offset=0;
	char * recPtr;
	objectType * thisSq;
	MemHandle h;
	
	levelType *level;
	
	
	
	if (DatabaseAccess_openDB(name, type, &m_db))
	{
		level = MemPtrNew(sizeof(levelType));
		if (level != NULL)
		{
			char * levelName;
			offset = 0;	
			h = DmQueryRecord(m_db, index);
			
			if (h)
			{
				recPtr = MemHandleLock(h);
				
				// read level name
				StrCopy(level->levelName, recPtr);
				recPtr+=StrLen(recPtr)+1; // in lieu of offset
				
				// read # non null squares
				nonnull = StrAToI(recPtr);
				recPtr+=StrLen(recPtr)+1;
				
				// read levels index in levels db
				level->index = StrAToI(recPtr);
				recPtr+=StrLen(recPtr)+1;
				
				// load dimension
				level->dimension = StrAToI(recPtr);
				recPtr+=StrLen(recPtr)+1;
				
				// run this now because it sets the values for MAX_X_SQUARES etc.
				setGameDimensions(level->dimension);
				
				// load the hint
				StrCopy(level->levelHint, recPtr);
				recPtr += StrLen(recPtr)+1;
				
				// init squares
				for (x=0; x<MAX_X_SQUARES; x++)
					for (y=0; y<MAX_Y_SQUARES; y++)
						level->square[x][y] = NULL;
				
				// load up the squares
				for (i=0; i<nonnull; i++)
				{
					thisSq = loadLevel_loadSquare(&recPtr);
					
					if (thisSq != NULL)
					{
						level->square[thisSq->position.x][thisSq->position.y] = thisSq;
					}
				}
				
				// create the perimeter
				for (x=0; x<MAX_X_SQUARES; x++)
				{
					level->square[x][0] = create_object(SOLID_SQUARE, x, 0, NO_DIR);
					level->square[x][MAX_Y_SQUARES-1] = create_object(SOLID_SQUARE, x, MAX_Y_SQUARES-1, NO_DIR);
				}
				
				for (y=1; y<MAX_Y_SQUARES-1; y++)
				{
					level->square[0][y] = create_object(SOLID_SQUARE, 0, y, NO_DIR);
					level->square[MAX_X_SQUARES-1][y] = create_object(SOLID_SQUARE, MAX_X_SQUARES-1, y, NO_DIR);
				}
				
				for (x=1; x<MAX_X_SQUARES-1; x++)
					for (y=1; y<MAX_Y_SQUARES-1; y++)
						if (level->square[x][y] != NULL)
							if (level->square[x][y]->type == TRACK_MOVE)
							{
								int xt, yt;
								objectType * thisSq = level->square[x][y];
								
								// record the position of the gun
								xt = thisSq->extras.trackArrow.trackGun->position.x;
								yt = thisSq->extras.trackArrow.trackGun->position.y;
								
								// delete the gun tmp
								MemPtrFree(thisSq->extras.trackArrow.trackGun);
								
								//reassign to the correct square
								thisSq->extras.trackArrow.trackGun = level->square[xt][yt];
								
							}
					
				MemHandleUnlock(h);
				success = true;	
			}
			else
			{
				Err i = DmGetLastErr();
				printf ("%s", i);
			}
		
			DmCloseDatabase(m_db);
			
		}
	
	}
	
	gameState->level = level;
	gameState->levelFinished = false;
	
	return success;
}

objectType * loadLevel_loadSquare(char ** recPtrN)
{
	objectType *thisSq = MemPtrNew(sizeof(objectType));
	char * recPtr = *recPtrN;
	
	if (thisSq != NULL)
	{
		// read in definites
		thisSq->position.x = StrAToI(recPtr);
		recPtr+=StrLen(recPtr)+1;
		
		thisSq->position.y = StrAToI(recPtr);
		recPtr+=StrLen(recPtr)+1;
		
		thisSq->type = StrAToI(recPtr);
		recPtr+=StrLen(recPtr)+1;
		
		thisSq->direction = StrAToI(recPtr);
		recPtr+=StrLen(recPtr)+1;
		
		
		switch (thisSq->type)
		{
			case TRACK_MOVE:
			{
				// create temp object with valid position
				// resolve after loading
				objectType *tmpGun = MemPtrNew(sizeof(objectType));
				if (tmpGun != NULL)
				{
					tmpGun->position.x = StrAToI(recPtr);
					recPtr+=StrLen(recPtr)+1;
		
					tmpGun->position.y = StrAToI(recPtr);
					recPtr+=StrLen(recPtr)+1;
					
					thisSq->extras.trackArrow.trackGun = tmpGun;
				}
			}
		}
	}
	*recPtrN = recPtr;
	return thisSq;
}

int level_getNonNullSquares(levelType *level)
{
	int x,y, count=0;
	if (level != NULL)
	{
		for (x=1; x<MAX_X_SQUARES-1; x++)
			for (y=1; y<MAX_Y_SQUARES-1; y++)
				if (level->square[x][y] != NULL)
					count++;
	}
	
	return count;
}


/***************************************************************************
*
*	Menu handling functions
*
*
*
/***************************************************************************/

static Boolean menuInstructions_OnSelect(UInt16 menuID)
{
	FrmInitForm(frmInstructions);
	FrmDoDialog(FrmGetFormPtr(frmInstructions));
	FrmDeleteForm(FrmGetFormPtr(frmInstructions));
	drawScreen();
	return true;
}

static Boolean mnuHint_OnSelect(UInt16 menuID)
{
	if (gameState != NULL)
	{
		FrmCustomAlert(alHint, gameState->level->levelHint, "", "");
		drawScreen();
	}
	
	return true;
}

static Boolean mnuAbout_OnSelect(UInt16 menuID)
{
	FrmInitForm(frmAbout);
	FrmDoDialog(FrmGetFormPtr(frmAbout));
	FrmDeleteForm(FrmGetFormPtr(frmAbout));
	drawScreen();
	return true;
}

static Boolean mnuRestart_OnSelect(UInt16 menuID)
{
	int index = gameState->level->index;
	
	destroyCurrentLevel();
	destroyRemainingShots();
	DatabaseAccess_loadLevelIntoGameState(LEVELS_DB_NAME, LEVELS_DB_TYPE, index);
	drawScreen();
	
	return true;
}